<?php

namespace Lavalite\Example\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Database\RequestScope;
use Lavalite\Example\Forms\Example as ExampleForm;
use Lavalite\Example\Http\Requests\ExampleResourceRequest;
use Lavalite\Example\Http\Resources\ExampleResource;
use Lavalite\Example\Http\Resources\ExamplesCollection;
use Lavalite\Example\Models\Example;
use Lavalite\Example\Scopes\ExampleResourceScope;


/**
 * Resource controller class for example.
 */
class ExampleResourceController extends BaseController
{

    /**
     * Initialize example resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->form = ExampleForm::grouped(false)
                        ->setAttributes()
                        ->toArray();
        $this->modules = $this->modules(config('lavalite.example.modules'), 'example', guard_url('example'));
    }

    /**
     * Display a list of example.
     *
     * @return Response
     */
    public function index(ExampleResourceRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $page = Example::pushScope(new RequestScope())
            ->pushScope(new ExampleResourceScope())
            ->paginate($pageLimit);

        $data = new ExamplesCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('example::example.names'))
            ->view('example::example.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display example.
     *
     * @param Request $request
     * @param Model   $example
     *
     * @return Response
     */
    public function show(ExampleResourceRequest $request, Example $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ExampleResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('example::example.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('example::example.show')
            ->output();
    }

    /**
     * Show the form for creating a new example.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ExampleResourceRequest $request, Example $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ExampleResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('example::example.name'))
            ->view('example::example.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new example.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ExampleResourceRequest $request, Example $model)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $model = $model->create($attributes);
            $data = new ExampleResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('example::example.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('example/example/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/example/example'))
                ->redirect();
        }

    }

    /**
     * Show example for editing.
     *
     * @param Request $request
     * @param Model   $example
     *
     * @return Response
     */
    public function edit(ExampleResourceRequest $request, Example $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new ExampleResource($model);
        // return view('example::example.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('example::example.name'))
            ->view('example::example.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the example.
     *
     * @param Request $request
     * @param Model   $example
     *
     * @return Response
     */
    public function update(ExampleResourceRequest $request, Example $model)
    {
        try {
            $attributes = $request->all();
            $model->update($attributes);
            $data = new ExampleResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('example::example.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('example/example/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('example/example/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the example.
     *
     * @param Model   $example
     *
     * @return Response
     */
    public function destroy(ExampleResourceRequest $request, Example $model)
    {
        try {
            $model->delete();
            $data = new ExampleResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('example::example.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('example/example/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('example/example/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
